# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

from htag import Tag
import json

class TagWcSplyt(Tag.wc_splyt):

    # statics = Tag.script(_src="https://my.netlib.re/wyc/gen/splyt.py")  # WYC "web component" in python ;-)
    statics = [Tag.script(r"""var $jscomp=$jscomp||{};$jscomp.scope={};$jscomp.createTemplateTagFirstArg=function(c){return c.raw=c};$jscomp.createTemplateTagFirstArgWithRaw=function(c,p){c.raw=p;return c};
!function(c,p){"object"==typeof exports&&"undefined"!=typeof module?module.exports=p():"function"==typeof define&&define.amd?define(p):(c=c||self).Split=p()}(this,function(){var c="undefined"!=typeof window?window:null,p=null===c,x=p?void 0:c.document,r=function(){return!1},Y=p?"calc":["","-webkit-","-moz-","-o-"].filter(function(g){var e=x.createElement("div");return e.style.cssText="width:"+g+"calc(9px)",!!e.style.length}).shift()+"calc",O=function(g){if("string"==typeof g||g instanceof String){var e=
x.querySelector(g);if(!e)throw Error("Selector "+g+" did not match a DOM element");return e}return g},q=function(g,e,m){g=g[e];return void 0!==g?g:m},E=function(g,e,m,t){if(e){if("end"===t)return 0;if("center"===t)return g/2}else if(m){if("start"===t)return 0;if("center"===t)return g/2}return g},Z=function(g,e){var m=x.createElement("div");return m.className="gutter gutter-"+e,m},aa=function(g,e,m){var t={};return"string"==typeof e||e instanceof String?t[g]=e:t[g]=Y+"("+e+"% - "+m+"px)",t},ba=function(g,
e){var m;return(m={})[g]=e+"px",m};return function(g,e){function m(d,a,b,f){var l=P(y,a,b,f);Object.keys(l).forEach(function(h){d.style[h]=l[h]})}function t(){return n.map(function(d){return d.size})}function Q(d){var a=n[this.a],b=n[this.b],f=a.size+b.size;a.size=d/this.size*f;b.size=f-d/this.size*f;m(a.element,a.size,this._b,a.i);m(b.element,b.size,this._c,b.i)}function ca(d){var a,b=n[this.a],f=n[this.b];this.dragging&&(a=("touches"in d?d.touches[0][A]:d[A])-this.start+(this._b-this.dragOffset),
1<G&&(a=Math.round(a/G)*G),a<=b.minSize+R+this._b?a=b.minSize+this._b:a>=this.size-(f.minSize+R+this._c)&&(a=this.size-(f.minSize+this._c)),Q.call(this,a),q(e,"onDrag",r)(t()))}function S(){var d=n[this.b].element,a=n[this.a].element.getBoundingClientRect();d=d.getBoundingClientRect();this.size=a[y]+d[y]+this._b+this._c;this.start=a[H];this.end=a[I]}function T(d){var a=function(h){if(!getComputedStyle)return null;var k=getComputedStyle(h);if(!k)return null;h=h[J];return 0===h?null:h-("horizontal"===
B?parseFloat(k.paddingLeft)+parseFloat(k.paddingRight):parseFloat(k.paddingTop)+parseFloat(k.paddingBottom))}(C);if(null===a||K.reduce(function(h,k){return h+k},0)>a)return d;var b=0,f=[],l=d.map(function(h,k){var u=a*h/100,v=E(D,0===k,k===d.length-1,F);v=K[k]+v;return u<v?(b+=v-u,f.push(0),v):(f.push(u-v),u)});return 0===b?d:l.map(function(h,k){var u=h;0<b&&0<f[k]-b&&(u=Math.min(b,f[k]-b),b-=u,u=h-u);return u/a*100})}function da(){var d=n[this.a].element,a=n[this.b].element;this.dragging&&q(e,"onDragEnd",
r)(t());this.dragging=!1;c.removeEventListener("mouseup",this.stop);c.removeEventListener("touchend",this.stop);c.removeEventListener("touchcancel",this.stop);c.removeEventListener("mousemove",this.move);c.removeEventListener("touchmove",this.move);this.move=this.stop=null;d.removeEventListener("selectstart",r);d.removeEventListener("dragstart",r);a.removeEventListener("selectstart",r);a.removeEventListener("dragstart",r);d.style.userSelect="";d.style.webkitUserSelect="";d.style.MozUserSelect="";
d.style.pointerEvents="";a.style.userSelect="";a.style.webkitUserSelect="";a.style.MozUserSelect="";a.style.pointerEvents="";this.gutter.style.cursor="";this.parent.style.cursor="";x.body.style.cursor=""}function ea(d){if(!("button"in d)||0===d.button){var a=n[this.a].element,b=n[this.b].element;this.dragging||q(e,"onDragStart",r)(t());d.preventDefault();this.dragging=!0;this.move=ca.bind(this);this.stop=da.bind(this);c.addEventListener("mouseup",this.stop);c.addEventListener("touchend",this.stop);
c.addEventListener("touchcancel",this.stop);c.addEventListener("mousemove",this.move);c.addEventListener("touchmove",this.move);a.addEventListener("selectstart",r);a.addEventListener("dragstart",r);b.addEventListener("selectstart",r);b.addEventListener("dragstart",r);a.style.userSelect="none";a.style.webkitUserSelect="none";a.style.MozUserSelect="none";a.style.pointerEvents="none";b.style.userSelect="none";b.style.webkitUserSelect="none";b.style.MozUserSelect="none";b.style.pointerEvents="none";this.gutter.style.cursor=
L;this.parent.style.cursor=L;x.body.style.cursor=L;S.call(this);this.dragOffset=("touches"in d?d.touches[0][A]:d[A])-this.end}}function U(d){var a=d.i===z.length,b=a?z[d.i-1]:z[d.i];S.call(b);Q.call(b,a?b.size-d.minSize-b._c:d.minSize+b._b)}if(void 0===e&&(e={}),p)return{};var y,A,H,I,J,n,w=g;Array.from&&(w=Array.from(w));var C=O(w[0]).parentNode,V=getComputedStyle?getComputedStyle(C):null,W=V?V.flexDirection:null,M=q(e,"sizes")||w.map(function(){return 100/w.length}),N=q(e,"minSize",100),K=Array.isArray(N)?
N:w.map(function(){return N}),fa=q(e,"expandToMin",!1),D=q(e,"gutterSize",10),F=q(e,"gutterAlign","center"),R=q(e,"snapOffset",30),G=q(e,"dragInterval",1),B=q(e,"direction","horizontal"),L=q(e,"cursor","horizontal"===B?"col-resize":"row-resize"),ha=q(e,"gutter",Z),P=q(e,"elementStyle",aa),ia=q(e,"gutterStyle",ba);"horizontal"===B?(y="width",A="clientX",H="left",I="right",J="clientWidth"):"vertical"===B&&(y="height",A="clientY",H="top",I="bottom",J="clientHeight");M=T(M);var z=[];return(n=w.map(function(d,
a){var b,f={element:O(d),size:M[a],minSize:K[a],i:a};if(0<a&&((b={a:a-1,b:a,dragging:!1,direction:B,parent:C})._b=E(D,0==a-1,!1,F),b._c=E(D,!1,a===w.length-1,F),"row-reverse"===W||"column-reverse"===W)){var l=b.a;b.a=b.b;b.b=l}0<a&&(l=ha(a,B,f.element),!function(h,k,u){var v=ia(y,k,u);Object.keys(v).forEach(function(X){h.style[X]=v[X]})}(l,D,a),b._a=ea.bind(b),l.addEventListener("mousedown",b._a),l.addEventListener("touchstart",b._a),C.insertBefore(l,f.element),b.gutter=l);return m(f.element,f.size,
E(D,0===a,a===w.length-1,F),a),0<a&&z.push(b),f})).forEach(function(d){var a=d.element.getBoundingClientRect()[y];a<d.minSize&&(fa?U(d):d.minSize=a)}),{setSizes:function(d){var a=T(d);a.forEach(function(b,f){if(0<f){var l=z[f-1],h=n[l.a],k=n[l.b];h.size=a[f-1];k.size=b;m(h.element,h.size,l._b,h.i);m(k.element,k.size,l._c,k.i)}})},getSizes:t,collapse:function(d){U(n[d])},destroy:function(d,a){z.forEach(function(b){if(!0!==a?b.parent.removeChild(b.gutter):(b.gutter.removeEventListener("mousedown",b._a),
b.gutter.removeEventListener("touchstart",b._a)),!0!==d){var f=P(y,b.a.size,b._b);Object.keys(f).forEach(function(l){n[b.a].element.style[l]="";n[b.b].element.style[l]=""})}})},parent:C,pairs:z}}});var _pyfunc_truthy=function(c){return null===c||"object"!==typeof c?c:void 0!==c.length?c.length?c:!1:void 0!==c.byteLength?c.byteLength?c:!1:c.constructor!==Object?!0:Object.getOwnPropertyNames(c).length?c:!1},WcSplyt;
WcSplyt=function(){var c=Reflect.construct(HTMLElement,[],WcSplyt);if(c._template){var p=document.createElement("template");p.innerHTML=c._template;c.attachShadow({mode:"open"}).appendChild(p.content.cloneNode(!0))}c.init&&c.init();return c};WcSplyt.prototype=Object.create(HTMLElement.prototype);WcSplyt.prototype._base_class=HTMLElement.prototype;WcSplyt.prototype.__name__="WcSplyt";WcSplyt.prototype.init=function(){this.root=this.shadowRoot.querySelector("#split");return this._s=null};
WcSplyt.prototype.modeChanged=function(){this.connectedCallback();this.dispatchEvent(new window.Event("change"));return null};WcSplyt.prototype.sizesChanged=function(){var c=this.getAttribute("sizes");_pyfunc_truthy(this._s)&&_pyfunc_truthy(c)&&(this._s.setSizes(JSON.parse(c)),this.dispatchEvent(new window.Event("change")));return null};
WcSplyt.prototype.connectedCallback=function(){var c=_pyfunc_truthy(this.getAttribute("mode"))||"horizontal";var p=JSON.parse(_pyfunc_truthy(this.getAttribute("sizes"))||"[50, 50]");this.root.className=c;_pyfunc_truthy(this._s)&&this._s.destroy();this._s=new Split(this.shadowRoot.querySelectorAll("slot"),{direction:c,sizes:p,minSize:[0,0],onDragEnd:function(x){return this.changeSize(x)}.bind(this)});return null};
WcSplyt.prototype.changeSize=function(c){this.setAttribute("sizes",JSON.stringify(c));this.dispatchEvent(new window.Event("change"));return null};Object.defineProperty(WcSplyt,"observedAttributes",{get:function(){return["mode","sizes"]}});WcSplyt.prototype._template='\n<style>\nslot {display:block}\n\n#split {\n    width:100%;\n    height:100%;\n}\n#split.horizontal {\n    display: flex;\n}\n#split.vertical {\n}\n\n.gutter {\n    background-color: #eee;\n    background-repeat: no-repeat;\n    background-position: 50%;\n}\n\n.gutter.gutter-horizontal {\n    background-image: url(\'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAUAAAAeCAYAAADkftS9AAAAIklEQVQoU2M4c+bMfxAGAgYYmwGrIIiDjrELjpo5aiZeMwF+yNnOs5KSvgAAAABJRU5ErkJggg==\');\n    cursor: col-resize;\n}\n\n.gutter.gutter-vertical {\n    background-image: url(\'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAB4AAAAFAQMAAABo7865AAAABlBMVEVHcEzMzMzyAv2sAAAAAXRSTlMAQObYZgAAABBJREFUeF5jOAMEEAIEEFwAn3kMwcB6I2AAAAAASUVORK5CYII=\');\n    cursor: row-resize;\n}\n\n</style>\n<div id="split">\n    <slot name="a"></slot>\n    <slot name="b"></slot>\n</div>\n\n    ';
WcSplyt.prototype._reacts={mode:["modeChanged"],sizes:["sizesChanged"]};WcSplyt.prototype.attributeChangedCallback=function(c,p,x){if(this._reacts[c])for(var r in this._reacts[c])this[this._reacts[c][r]]()};customElements.define("wc-splyt",WcSplyt);""")]  # WYC "web component" in python ;-)


    def changeSize(self,v):
        # save the rendering size (to redo the same sizes !)
        self["sizes"] = [float(i) for i in json.loads(v)]

class HSplit(TagWcSplyt):
    def __init__(self,left,right, sizes=[50,50],**a):
        super().__init__(**a)
        self["sizes"] = sizes
        self["mode"] = "horizontal"
        # when sizes change, ensure to inform the py component
        self["onchange"] = self.bind.changeSize( b"this.getAttribute('sizes')")
        self <= Tag.div(left,_slot="a",_style="height:100%")
        self <= Tag.div(right,_slot="b",_style="height:100%")

class VSplit(TagWcSplyt):
    def __init__(self,up,down, sizes=[50,50],**a):
        super().__init__(**a)
        self["sizes"] = sizes
        self["mode"] = "vertical"
        self["onchange"] = self.bind.changeSize( b"this.getAttribute('sizes')")
        self <= Tag.div(up,_slot="a",_style="height:100%")
        self <= Tag.div(down,_slot="b",_style="height:100%")


if __name__=="__main__":
    v1=VSplit("top1","bottom1")
    v2=VSplit("top2","bottom2",sizes=[30,70])
    obj= HSplit(v1,v2)

    from . import _test
    _test( obj )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from . import TagBulma
from htag import Tag

class Tags(TagBulma):
    tag="div"
    """
        tags:       mutable list of tags (should be in choices)
        choices:    complete list of available tags
        editable :  can edit the tags or not
    """

    def __init__(self, tags: list, choices: list, editable=True,**a):
        super().__init__(**a)
        self.tags = tags
        self.editable = editable
        self.choices = choices
        self.update()

    def update(self):
        self.clear()
        for i in self.choices:
            if self.editable:
                klass = "tag is-success" if i in self.tags else "tag"
                self.add(
                    Tag.span(
                        i,
                        _class=klass,
                        _onclick=self.bind.switch(i),
                        _style="cursor:pointer",
                    )
                )
            else:
                if i in self.tags:
                    self.add(Tag.span(i, _class="tag is-success"))
            self.add(" ")

    def switch(self, t):
        if t in self.tags:
            self.tags.remove(t)
        else:
            self.tags.append(t)
        self.update()

if __name__=="__main__":

    ll = ["banana", "apple", "pear", "peach", "melon", "cherry", "plum"]
    obj=Tags( [ll[3],ll[5]], ll)

    from . import _test
    _test( obj )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from .. import TagBulma,Button,Box,HBox,Content
from htag import Tag

class MBox(TagBulma):
    tag="div"

    imports = Button,Box,HBox,Content

    def __init__(self, parent):
        """ auto attach on 'parent' """
        super().__init__()
        print("**DEPRECATED** don't use MBox, use the new Service !")
        self["info"]="mbox"
        parent.add( self )

    def show(self, content, canClose=True, full=False):
        self.clear()
        o = Tag.div(_class = "modal is-active")

        jsclose= self.bind.close()

        bg=Tag.div(_class="modal-background")
        o <= bg
        if canClose:
            bg["onclick"]=jsclose
            o <= Tag.div(
                    _class="modal-close is-large",
                    _aria_label="close",
                    _onclick=jsclose,
                )

        o <= Tag.div( Box( content , _style="height:100%;overflow-y:auto" if full else None) , _class="modal-content", _style = "width:90%;height:98%;" if full else None )


        self.add( o )

    def confirm(self, content, ok, ko=None,txtok="OK",txtko="Cancel"):
        self._cbok = ok
        self._cbko = ko
        js = """if (event.keyCode === 27) {event.preventDefault();%s;}""" % self.bind._confirm(0)

        main = Content( content )
        main <= HBox(
            Tag.div(_style="flex: 1 0 25%;"),
            Tag.div(_style="flex: 1 0 25%;"),
            Button(txtko, _onclick=self.bind._confirm(0), _class="is-light",_style="flex: 1 0 25%;"),
            Button(txtok, _onclick=self.bind._confirm(1), js="self.focus()",_onkeyup = js , _style="flex: 1 0 25%;"),
        )
        self.show(main, canClose=True)

    def _confirm(self, ok):
        self.close()
        if ok and self._cbok:
            self._cbok()
        elif self._cbko:
            self._cbko()


    def prompt(self, title, defaultValue, ok, ko=None,txtok="OK",txtko="Cancel"):
        self._cbok = ok
        self._cbko = ko

        js =  """if (event.keyCode === 13) {event.preventDefault();%s;}""" % self.bind._prompt(b"this.value")
        js += """if (event.keyCode === 27) {event.preventDefault();%s;}""" % self.bind._prompt()
        input = Tag.input(_value=defaultValue, js="self.focus();self.setSelectionRange(0, self.value.length)", _class="input", _onkeyup = js)

        main = Content( Tag.h3(title) )
        main <= input
        main <= HBox(
            Tag.div(_style="flex: 1 0 25%;"),
            Tag.div(_style="flex: 1 0 25%;"),
            Button(txtko, _onclick=self.bind._prompt(), _class="is-light",_style="flex: 1 0 25%;"),
            Button(txtok, _onclick=self.bind._prompt( b"document.getElementById('%d').value" % id(input)) ,_style="flex: 1 0 25%;"),
            _style="margin-top:10px"
        )
        self.show(main, canClose=True)

    def _prompt(self, value=None):
        self.close()
        if (value is not None) and self._cbok:
            self._cbok(value)
        elif self._cbko:
            self._cbko()


    def close(self):
        self.clear()



if __name__=="__main__":
    def majok(v=None):
        print("ok",v)
    def majko():
        print("ko")

    obj=Tag.div( )
    MBox(obj).show( Content("YO"), canClose=False )
    # MBox(obj).show( Content("YO"), full=True )
    # MBox(obj).confirm("hello ? sure ?????", ok=majok, ko=majko)
    # MBox(obj).prompt("What's your name ?","john doe", ok=majok, ko=majko)

    from .. import _test
    _test( obj )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from htag import Tag
import ast,json
# from htag.tag import Caller
# import json,html
from htbulma import TagBulma, Form

from typing import Union

ListOrDict = Union[ list, dict]



#########################################################################################################################
## Common helpers
#########################################################################################################################

class Options:

    def __init__(self,options:ListOrDict):
        if isinstance(options,list):
            self._options = {i:i for i in options}
        elif isinstance(options,dict):
            self._options = dict(options)
        else:
            self._options={}

    def fix_value_if_options(self,value): #TODO: dont work for float,bool,.. (only int/str) ;-(
        if self._options:
            if isinstance(value, str) and value.isnumeric() and int(value) in self._options.keys():
                return int(value)
        return value


class NewInput:

    _value=None
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = value
        # and draw
        self._redraw()

    def _redraw(self):
        pass


#########################################################################################################################
## Generics (not bulma related)
#########################################################################################################################

class Input(Tag.input,Options,NewInput):
    statics= """.disabled {pointer-events:none;cursor: not-allowed;opacity:0.5}"""

    """
    a.keys() can be :
        _type : date,email,color,file etc ...
        _required
        _readonly
        _disabled
        _style
        _class
        _placeholder
        ...
    """

    def __init__(self, value, options:ListOrDict=[], name=None, **a):
        Options.__init__(self,options)
        super().__init__(**a)
        self.value=value  # store the real value in a property value

        self["class"].add("input")
        self["onchange"] = self.bind( self._set, b"this.value" ).bind( a.get("_onchange",None) )    # assign an event to reflect change

        if name: self["name"]=name

        if self._options:
            self["type"]="search"
            datalist = Tag.datalist( [Tag.option(v,_value=k) for k,v in self._options.items()] )
            self <= datalist
            self["list"] = id(datalist)

    def _redraw(self):
        self["value"]=self._value

    def _set(self,value:str): # when changed, keep the property value up-to-date
        self._value = self.fix_value_if_options(value)


class Select(Tag.select,Options,NewInput):

    def __init__(self, value, options:ListOrDict,name=None,**a):
        Options.__init__(self,options)
        super().__init__(**a)

        self["class"].add("select")
        self["onchange"] = self.bind( self._set, b"this.value").bind( a.get("_onchange",None) )

        if name: self["name"]=name

        if self["readonly"] or self["disabled"]:
            self["class"].add("disabled")

        self.value = value  # redraw

    def _redraw(self):
        self["value"]=self._value
        self.clear()
        if self._options:
            for k,v in self._options.items():
                self <= Tag.option( v, _value=k, _selected = (self._value == k) )

    def _set(self,value:str):
        self._value  = self.fix_value_if_options(value)


class Checkbox(Tag.input,NewInput):

    #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
    #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ overrides !
    #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = bool(value)
        # and draw
        """
        **WARNING TRICK ***********************************************************************************
        returning "<input><label>xxx</label></input>" will produce "<input/> <label>xxx</label>" in dom
        so labels will stack, at each change ;-(
        ===> the self.clear() avoid that !!!!!!!!!!!!!!!
        **WARNING TRICK ***********************************************************************************
        """
        self.clear()
        self["checked"]=self._value
    #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
    #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
    #/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\

    def __init__(self, value:bool, label=None, name=None,**a):
        super().__init__(**a)
        self.value = value  # !important here

        self["class"].add("checkbox")
        self["type"]="checkbox"
        self["onchange"] = self.bind( self._set, b"this.checked").bind( a.get("_onchange",None) )

        if self["readonly"] or self["disabled"]:
            self["class"].add("disabled")

        if name: self["name"]=name

        if label:
            self <= Tag.label(label,_for=str(id(self)),_class="disabled" if self["readonly"] or self["disabled"] else "")

    def _set(self,value:str):
        self._value = value in ["true","on","yes",True,1]


class Range(Tag.input,NewInput):

    def __init__(self, value, name=None,**a):
        super().__init__(**a)

        self["type"]="range"
        self["class"].add("control")
        if name: self["name"]=name

        if self["readonly"]:
            self["class"].add("disabled")

        self["onchange"] = self.bind( self._set, b"this.value" ).bind( a.get("_onchange",None) )

        self.value = value

    def _redraw(self):
        self["value"]=self._value

    def _set(self,value):
        self._value = ast.literal_eval(value)


class Textarea(Tag.textarea,NewInput):

    def __init__(self, value:str, name=None,**a):
        super().__init__(**a)

        self["class"].add("textarea")
        self["onchange"] = self.bind( self._set, b"this.value" ).bind( a.get("_onchange",None) )    # assign an event to reflect change

        if self["readonly"]:
            self["class"].add("disabled")

        if name: self["name"]=name
        self.value = value

    def _redraw(self):
        self.set( self._value )

    def _set(self,value:str):
        self._value = value


class Radio(Tag.span,Options,NewInput):

    def __init__(self, value, options:ListOrDict, name=None,**a):
        Options.__init__(self,options)
        super().__init__(**a)

        self["class"].add("control")
        self["onchange"] = self.bind( a.get("_onchange",lambda o:None) )   # TODO: evol htag ?! bcoz 2 interactions

        if self["readonly"] or self["disabled"]:
            self._disabled=True
            self["class"].add("disabled")
        else:
            self._disabled=False

        self._default_name = name or ("r%s" % id(self))

        self.value=value    # redraw

    def _redraw(self):
        self.clear()
        if self._options:
            for k,v in self._options.items():
                i=Tag.input(_type="radio", _value=k, _checked = (self._value==k), _name = self._default_name,_disabled=self._disabled )
                i["onchange"] = self.bind( self._set, b"this.value" )
                self <= Tag.label( [i," ",v], _class="radio" )


    def _set(self,value):
        self._value = self.fix_value_if_options(value)

#########################################################################################################################
## Bulma specifics
#########################################################################################################################

class SelectButtons(Tag.div,Options,NewInput):
    _bstyle_ = "is-toggle is-small"

    def __init__(self, value, options:ListOrDict, name=None,**a):
        Options.__init__(self,options)
        super().__init__(**a)

        self["class"].add("tabs",self._bstyle_)
        self["onchange"] = self.bind( a.get("_onchange",lambda o:None) )   # TODO: evol htag ?! bcoz 2 interactions

        self.input = Tag.input(_name=name,_type="hidden",_value=self._value, **a)
        self.u = Tag.ul()

        self<= self.input + self.u

        self.value=value

    def _redraw(self):
        self.input["value"]=self._value
        self.u.clear()
        if self["disabled"] or self["readonly"]:
            self.u["class"].add("disabled")

        for k,v in self._options.items():
            isActive = "is-active" if self._value == k else ""
            if self["disabled"]:
                self.u<=Tag.li(Tag.a(v,_disabled=True), _class=isActive, _disabled=True)
            else:
                self.u<=Tag.li(Tag.a(v, _onclick=self.bind(  self._set, k )), _class=isActive)

    def _set(self,value):
        self._value = self.fix_value_if_options(value)
        self.call( self["onchange"] )
        self._redraw()

class TabsHeader(SelectButtons):
    _bstyle_="is-centered" # "is-boxed is-centered"




# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from . import TagBulma
from htag import Tag


class Fields(Tag.fieldset,TagBulma):
    """ Just a helper to produce beautiful bulma forms """
    def __init__(self, **a): # accept bool "_disabled" !!!
        super().__init__(**a)

    def addField(self, label, obj,info=None):
        left = Tag.div( Tag.div(label,_class="label"), _class="field-label is-normal")
        contents=[Tag.div(obj,_class="control")]
        if info: contents.append( Tag.p(info,_class="help") )
        right = Tag.div( Tag.div( contents,_class="field"), _class="field-body")
        self <= Tag.div( [left,right], _class="field is-horizontal")

if __name__=="__main__":


    nav= Fields()
    nav.addField( "zone1 ", "object1" )
    nav.addField( "zone2 ", "object2", "help2" )

    from . import _test
    _test( nav )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from . import TagBulma,TabsHeader

class Tabs(TagBulma): # New version (NOT htag optimized ;-()

    imports = TabsHeader

    def __init__(self,**a):
        super().__init__(**a)
        self.__tabs={}
        self.__selected = None

    def addTab(self,name,content):
        self.__tabs[name] = content
        if self.__selected is None:
            self.__selected=name
        self._render()

    def _render(self):
        self.clear()

        ll=[]
        for k,v in self.__tabs.items():
            ll.append(k)

        if self.__selected:
            if len(ll)>0:
                self <= TabsHeader(self.__selected, ll, _onchange= self._setselected )
                try: # to be compatible with htag>0.8.9
                    #TODO: remove when new htag is widely deployed
                    self.add( self.__tabs[ self.__selected ], True) # reparenting
                except:
                    self.add( self.__tabs[ self.__selected ] )

    @property
    def selected(self):
        return self.__selected

    @selected.setter
    def selected(self, v):
        self.__selected =v
        self._render()

    def _setselected(self,obj):
        self.__selected = obj.value
        self._render()

if __name__=="__main__":

    obj=Tabs()
    obj.addTab("P1", "I'm the page1")
    obj.addTab("P2", "Currently, I am the page2 !")

    from . import _test
    _test( obj )




# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from .. import TagBulma,Box
from htag import Tag


class Toaster(TagBulma):
    tag="div"
    statics=[]

    def __init__(self, parent):
        """ auto attach on 'parent' """
        super().__init__()
        print("**DEPRECATED** don't use Toaster, use the new Service !")
        self["info"]="toaster"
        parent.add( self )

    def show(self,content,delay=2000): #TODO: can't be called immediatly ;-( (coz __caller__ for js interact)
        self.clear()

        jsclose= self.bind.close()

        o = Tag.div( _class="notification has-text-light has-background-grey",_style = "position:fixed;left:0px;right:0px;bottom:0px;z-index:1000")
        o.add( Tag.button(_class="delete", _onclick=jsclose) )
        o.add( content )

        self.call("""setTimeout(function() {%s;},%s);""" % (jsclose,delay))

        self <= o

    def close(self):
        self.clear()


if __name__=="__main__":

    # not really interactive in this test, so I need to put this
    # and not produce JS (close timeout)
    Toaster.__call__ = lambda x,y:x

    obj=Tag.div( )
    Toaster(obj).show( Box("Hello") )

    from .. import _test
    _test( obj )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

"""
Only containers can contain multiple objects ...
(like Box(*obj))

"""

from htag import Tag
from . import TagBulma,Button


class Flex(Tag.div):
    """ kind of hbox/vbox, but maximized (w/h @ 100%, without overflow).
        so, contents are overflow'able
    """
    def __init__(self,orientation:str,*objs,**k):
        assert orientation in ["row","column"]
        Tag.div.__init__(self,**k)
        self["style"]=f"display: flex;flex-flow: {orientation} nowrap;align-items:center;align-items: stretch;width:100%;height:100%;overflow:hidden"

        if objs:
            objs=list(objs)
            first=objs.pop(0)
            self.addNormal(first)
            for obj in objs:
                self.addFluid(obj)

    def append(self,obj,flex="1 0 auto"):   # shrinked by default !
        self <= Tag.div( obj, _style=f"flex:{flex};overflow:auto;")

    def addNormal(self,obj):
        """ shortcut """
        self.append(obj,"1 0 auto")
    def addFluid(self,obj):
        """ shortcut """
        self.append(obj,"1 1 auto")


class VFlex(Flex):
    """ vertical flex, first object keep its sizes, others expand (ideal to build a layout)"""
    def __init__(self,*objs,**attrs):
        Flex.__init__(self,"column",*objs,**attrs)


class HFlex(Flex):
    """ horizontal flex, first object keep its sizes, others expand (ideal to build a layout)"""
    def __init__(self,*objs,**attrs):
        Flex.__init__(self,"row",*objs,**attrs)



if __name__=="__main__":


    o=VFlex("TOP TITLE", VFlex( Button("BIG"),  HFlex( [Tag.div("MENU OP") for i in range(100)],"hello "*10000)) )

    from . import _test
    _test( o )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from htag import Tag

import json

"""
The latest (most recent) object (happy christmas 2022 !)
(it's coded using the latest tricks of htag (can be a reference, for future!))

MORE:
    - one interaction only
    - js method to force save, in js/clientside (generated by self.eventSave())
LESS:
    - it's not like others "inputs" standards here ;-(. So this object doesn't work as a b.Textarea, for example ;-(
      (/!\ aka not _onchange, but onsave /!\ (could change in future))
"""

class RichText(Tag.div):
    """ Rich Text Editor, based on Quill : https://quilljs.com/
        can deal with "real html" or "quill's delta"

        @value           : can be html or delta
        @onsave          : if defined, call a python callback (cb(self)) on 'js.eventSave()'
        @edit            : can make editable or not (default : true), but not dynamic !!!
        @opts            : (list) toolbar opts (see quilljs)
        .value           : is dynamic
        .eventSave()     : create an js/method to update server side (with html content)
        .eventSave(True) : create an js/method to update server side (with DELTA content)
    """

    statics=Tag.link(_href="//cdn.quilljs.com/1.3.6/quill.snow.css",_rel="stylesheet")
    statics+=Tag.script(_src="//cdn.quilljs.com/1.3.6/quill.js",_type="text/javascript")

    def init(self,value:"str or delta", onsave:"cb(self)"=None, opts:list=None, edit:bool=True, **a):
        self.onsave = onsave
        if opts is None:
            opts = [
              [{ 'header': [1, 2, 3, False] }],
              [{ 'size': ['small', False, 'large', 'huge'] }],
              [{ 'color': [] }, { 'background': [] }],
              ['bold', 'italic', 'underline', 'strike'],
              ['link', 'code-block'],
              [{ 'list': 'ordered'}, { 'list': 'bullet' }],
              #~ [{ 'indent': '-1'}, { 'indent': '+1' }],
              [{ 'align': [] }],
              ['clean'],
            ]

        self.js="""
            self.ed = new Quill(self, {
              modules: {
                toolbar: %s,
              },
              readOnly: %s,
              theme: %s,
            });

            self.getValue = function(asDelta) {
                if(asDelta)
                    return self.ed.getContents();
                else
                    return self.ed.root.innerHTML;
            }

            """ % (
            edit and json.dumps(opts) or "false",
            edit and "false" or "true",
            edit and "'snow'" or "false",
            )
        self.setValue(value,init=True)


    @property
    def value(self):
        return self._value

    @value.setter
    def value(self,value):
        self.setValue(value)

    def setValue(self,value:"str or delta",init:bool=False):
        self._value=value
        if isinstance(value,str):
            if init:
                self.set( value )
            else:
                self.call( f"self.ed.setContents(self.ed.clipboard.convert(`{value}`) ,'silent');" )
        else:
            cmd=f"self.ed.setContents( {json.dumps(value)}, 'silent')"
            if init:
                self.js += cmd
            else:
                self.call( cmd )

    def eventSave(self,asJson=False) -> "js call":
        tag=f"document.getElementById('{id(self)}')".encode()
        if asJson:
            return self.bind( self._set, b"%s.getValue(true)" % tag)
        else:
            return self.bind( self._set, b"%s.getValue()" % tag)

    def _set(self, data:"str or delta"):
        self.value = data
        if self.onsave: self.onsave( self )
        # if self["onchange"]: self["onchange"](self)




# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

"""
Only containers can contain multiple objects ...
(like Box(*obj))

"""

from htag import Tag
from . import TagBulma

class VBox(TagBulma):
    statics = [Tag.style("""
.vbox {display: flex;flex-flow: column nowrap;}
.vbox > * {flex: 1 1 auto;margin:1px !important;}
""")]

    def __init__(self,*objs,**a):
        super().__init__(**a)
        self["class"].add("vbox")
        for o in objs:
            self <= o


class HBox(TagBulma):
    statics = [Tag.style("""
.hbox {display: flex;flex-flow: row nowrap;align-items:center}
.hbox > * {flex: 1 1 auto;margin:1px !important;}
""")]

    def __init__(self,*objs,**a):
        super().__init__(**a)
        self["class"].add("hbox")
        for o in objs:
            self <= o

class Box(TagBulma):
    def __init__(self,*objs,**a):
        super().__init__(**a)
        self["class"].add("box")
        for o in objs:
            self <= o


class Section(TagBulma):
    def __init__(self,*objs,**a):
        self.tag="section" # enforce the html tag (bicoz TagBulma is nativly a div)

        super().__init__(**a)
        self["class"].add("section")
        for o in objs:
            self <= o

if __name__=="__main__":
    obj=Section( )
    obj<= Box("Hello")
    obj<= HBox( Box(1),Box(2),Box(3) )
    obj<= VBox( Box(1),Box(2),Box(3) )

    from . import _test
    _test( obj )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################
from htag import Tag
from . import TagBulma

class Progress(TagBulma):

    def __init__(self,**a):
        self.tag="progress" # enforce the html tag (bicoz TagBulma is nativly a div)
        super().__init__(**a)
        self["class"].add("progress","is-dark")

class Loader(TagBulma):
    """ it's clearly not the best bulma way to got that ;-)"""
    def __init__(self,**a):
        self.tag="span" # enforce the html tag (bicoz TagBulma is nativly a div)
        super().__init__(None,**a)
        self["class"].add("button","is-loading")
        self["style"].set("border","0px !important")


class Content(TagBulma):

    def __init__(self,txt=None,**a):
        super().__init__(**a)
        self["class"].add("content")
        self <= txt


class Button(TagBulma):

    def __init__(self,txt=None,**a):
        self.tag="button" # enforce the html tag (bicoz TagBulma is nativly a div)
        super().__init__(**a)
        self["class"].add("button","is-link")
        self <= txt


class A(TagBulma):

    def __init__(self,txt,**a):
        self.tag="a" # enforce the html tag (bicoz TagBulma is nativly a div)
        super().__init__(**a)
        self["class"].add("a","is-link")
        self <= txt



if __name__=="__main__":
    obj=Tag.div( )
    obj<= Button("Hello")
    obj<= Button("Hello", _class="is-small")
    obj<= Button("Hello",_class="is-success")
    obj<= A("Hello",_href="#")
    obj <= Progress()
    obj <= Loader()

    from . import _test
    _test( obj )



# -*- coding: utf-8 -*-
# #############################################################################
# Copyright (C) 2022 manatlan manatlan[at]gmail(dot)com
#
# MIT licence
#
# https://github.com/manatlan/htbulma
# #############################################################################

from . import TagBulma
from htag import Tag
import json

class Form(Tag.form):
    def __init__(self,onsubmit=None,**a):
        Tag.__init__(self,**a)
        # rewrite the form.submit() (bicoz this method doesn't call the onsubmit ;-( )
        self.js = """self.submit=function() {%s}""" % self.bind._onsubmit(b"JSON.stringify(Object.fromEntries(new FormData(this)))")

        self["onsubmit"]="event.preventDefault();this.submit()"
        self._callback = onsubmit

    def _onsubmit(self,f:dict):
        if self._callback:
            self._callback(json.loads(f))


if __name__=="__main__":
    def onsubmit(form: dict):
        print(form)

    obj=Form(onsubmit=onsubmit)
    obj<=Tag.input(_name="txt1",_value="1", _class="input")
    obj<=Tag.input(_name="txt2",_value="2", _class="input")
    obj<=Tag.input(_name="txt3",_value="3", _class="input")
    obj<=Tag.input(_type="submit",_value="ok", _class="button")

    from . import _test
    _test( obj )
